/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.executable.gamepad;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.platform.raspberry.generator.executable.framework.ISingletonInitializationExecutable;
import org.fortiss.af3.platform.raspberry.generator.executable.framework.ISingletonTerminationExecutable;
import org.fortiss.af3.platform.raspberry.generator.executable.library.PiHALLibraryExecutableBase;

/**
 * Base class for all executables of the gamepad input sensors.
 * 
 * @author hoelzl
 */
abstract class GamepadExecutableBase<T extends EObject> extends PiHALLibraryExecutableBase<T>
		implements ISingletonInitializationExecutable, ISingletonTerminationExecutable {
	/** Constructor. */
	public GamepadExecutableBase(T modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonVariableDeclarationCode(String singletonPrefix) {
		return "static gamepad_configuration_t* " + singletonPrefix + "_gamepad_config;\n";
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonInitializationCode(String singletonPrefix) {
		String prefixedVar = singletonPrefix + "_gamepad_config";
		StringBuilder sb = new StringBuilder();
		sb.append("// initialize the gamepad configuration\n");
		sb.append(prefixedVar + " = malloc(sizeof(gamepad_configuration_t));\n");
		sb.append(prefixedVar + "->device_id = \"/dev/input/js0\";\n");
		sb.append(prefixedVar + "->waiting_sleep_in_micros = 250;\n");
		sb.append(prefixedVar + "->axis_callback = NULL;\n");
		sb.append(prefixedVar + "->button_callback = NULL;\n");
		sb.append("gamepad_initialize(" + prefixedVar + ");\n\n");
		return sb.toString();
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonTerminationCode(String singletonPrefix) {
		return "gamepad_terminate();\n";
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonIdentifier() {
		// use header file as singleton identifier
		return getHeaderFileName();
	}

	/** {@inheritDoc} */
	@Override
	public final String getHeaderFileName() {
		return "gamepad.h";
	}
}
